local Set = require("set")
Escritor = {}
Escritor.__index = Escritor

function Escritor.new()
	local escritor = {}
	setmetatable(escritor,Escritor)

	return escritor
end

--===================================================================
-- Funções de formatação de números reais

-- rounds a number to the nearest decimal places
--
function round(val, decimal)
  if (decimal) then
    return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
  else
    return math.floor(val+0.5)
  end
end

-- given a numeric value formats output rounded to given decimal places
--
function format_num(amount, decimal, prefix, neg_prefix)
  local str_amount,  formatted, famount, remain

  decimal = decimal or 2  -- default 2 decimal places
  neg_prefix = neg_prefix or "-" -- default negative sign

  famount = math.abs(round(amount,decimal))
  famount = math.floor(famount)

  remain = round(math.abs(amount) - famount, decimal)

        -- comma to separate the thousands
  --formatted = comma_value(famount)

        -- attach the decimal portion
  if (decimal > 0) then
    remain = string.sub(tostring(remain),3)
    formatted = famount .. "." .. remain ..
                string.rep("0", decimal - string.len(remain))
  end

        -- attach prefix string e.g '$' 
  formatted = (prefix or "") .. formatted 

        -- if value is negative then format accordingly
  if (amount<0) then
    if (neg_prefix=="()") then
      formatted = "("..formatted ..")"
    else
      formatted = neg_prefix .. formatted 
    end
  end

  return formatted
end

compRevisor = function (lhs, rhs)
	return lhs:getNome() < rhs:getNome()
end


compArtigo = function (lhs, rhs)
	if lhs:getMediaArtigo() > rhs:getMediaArtigo() then
		return true
	elseif lhs:getMediaArtigo() < rhs:getMediaArtigo() then
		return false
	else
		return lhs:getTitulo() < rhs:getTitulo()
	end
end

function Escritor:geraResumo(relatResumo, revista, edAtual)
	local meses = {"Janeiro","Fevereiro","Março","Abril","Maio","Junho","Julho","Agosto","Setembro","Outubro","Novembro","Dezembro"}
	local mes = string.match(edAtual:getDataPubl(), "%d+/(%d+)/%d+")
	local mesPublicacao = meses[tonumber(mes)]
	relatResumo:write("EngeSoft, num. ", tostring(edAtual:getNumero()), ", volume " , tostring(edAtual:getVolume())," - " , 
							mesPublicacao , 
							" de " , string.match(edAtual:getDataPubl(), "%d+/%d+/(%d+)"), "\n")
	relatResumo:write("Tema: ", (edAtual:getTema()):getNome(),"\n");
	if(edAtual:getEditorChefe() ~= nil) then
		relatResumo:write("Editor-chefe: ", (edAtual:getEditorChefe()):getNome())
	else
		relatResumo:write("Editor-chefe: ");
	end
	relatResumo:write("\n\n");
	
	local count = 0
	for i,_ in pairs(edAtual:getArtigos()) do
		count = count + 1
	end
	relatResumo:write("Artigos submetidos: ", count, "\n");		
	
	count = 0
	for i,_ in pairs((edAtual:getTema()):getCapacitados()) do
		count = count + 1
	end
	relatResumo:write("Revisores capacitados: ", count, "\n");
			
	--[[
	Verifica o número de revisores que efetivamente
	revisaram artigos da edição atual.
	]]--
	local revisores = Set.new();
	for _,art in pairs( edAtual:getArtigos() ) do
		for _,rev in pairs( art:getRevisoes() ) do
			revisores:insert(rev:getRevisor())
		end
	end
	relatResumo:write("Revisores envolvidos: ", #revisores, "\n");
			
	--[[
	Escreve a média artigos/revisor.
	]]--
	local somaArtigosRev = 0
	for _,rev in pairs( revisores ) do
		somaArtigosRev = somaArtigosRev + #rev:getRevisoes()
	end
	
	local media = tostring(format_num(somaArtigosRev/#revisores, 2))
	relatResumo:write("Média artigos/revisor: ", string.gsub(media, "%p", ","));
	relatResumo:close()
end

function Escritor:relatorioRevisoes(relatRevisoes, revista,edAtual)
	--[[
	* Ordena o conjunto de artigos da edição atual
	* de acordo com a média de cada. (Se as médias forem iguais, 
	* são ordenados de acordo com os seus títulos).
	--]]
	local artigosOrdenados = edAtual:getArtigos()
	table.sort(artigosOrdenados,compArtigo)
	
	for _,art in pairs( artigosOrdenados ) do
			
		--[[
		Lista para ordenar alfabeticamente os revisores de um artigo.
		]]--
		local revOrdenados = Set.new();
		
		for _,revisao in pairs( art:getRevisoes() ) do
			revOrdenados:insert(revisao:getRevisor())
		end
		
		table.sort(revOrdenados,compRevisor)
				
		relatRevisoes:write(art:getTitulo(), ";")
		--[[
		Escreve o nome do autor contato do artigo.
		]]--
		for _,aut in pairs( art:getAutorias() ) do
			if aut:isContato() then
				relatRevisoes:write((aut:getAutor()):getNome(), ";")
			end
		end
					
		--[[
		Escreve o código do artigo no padrão decimal com vírgula.
		]]--
		local media = tostring(format_num(art:getMediaArtigo(), 2))
		relatRevisoes:write(string.gsub(media,"%p",","))
				
		for i,r in pairs( revOrdenados ) do				
			relatRevisoes:write(";",r:getNome())
		end
		relatRevisoes:write("\n")
	end
	relatRevisoes:close();
end

function Escritor:relatorioRevisores(relatRevisores, revista,edAtual)
			
	--[[
	Conjunto para ordenar alfabeticamente os revisores de um artigo.
	]]--
	local revisores = Set.new();			
	for j,art in pairs( edAtual:getArtigos()) do
		for k,rev in pairs(art:getRevisoes()) do
			revisores:insert(rev:getRevisor())
		end
	end
	table.sort(revisores,compRevisor)
			
	for i,r in pairs(revisores) do
		local nRevisoes = 0;
		relatRevisores:write(r:getNome(),";")
		--[[
		Verifica o número de revisões que um revisor
		fez para a edição atual.
		]]--
		for j,art in pairs( edAtual:getArtigos()) do
			for k,rev in pairs(art:getRevisoes()) do
				if rev:getRevisor() == r then
					nRevisoes = nRevisoes + 1
				end
			end
		end
				
				
		--[[
		Escreve o código do artigo no padrão decimal com vírgula.
		]]--
		local media = tostring(format_num(r:getMediaRevisor(edAtual),2))
		relatRevisores:write(nRevisoes,";",string.gsub(media,"%p",","), "\n")
	end
	relatRevisores.close();
end
